/*
** Module   : UpdateAccountTask.java
** Abstract : a Runnable object used update temporary accounts status.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131216 Created initial version
*/

package com.goldencode.p2j.main;

import java.util.logging.*;

import com.goldencode.p2j.admin.*;
import com.goldencode.p2j.security.*;
import com.goldencode.p2j.security.SecurityManager;
import com.goldencode.p2j.util.LogHelper;
import com.goldencode.util.*;

/**
 * Runnable object used to update temporary accounts status.
 * This object is used as parameter to be executed by the temporary accounts pool thread.  
 * After the account is updated the security cache must be refreshed.
 * Otherwise the changes are not visible and the account remains disabled on later calls. 
 * On (re)enable account change also the password.  
 *  
 * @author mag
 *
 */
public class UpdateAccountTask 
implements Runnable
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(UpdateAccountTask.class.getName());
   
   /** Account pool item */
   private TemporaryAccount item;

   /** Enabled parameter */
   private boolean enabled;
     
   /**
    * Constructor.  
    * 
    * @param   item
    *          Account pool item.
    * @param   enabled
    *          Account status to be set.
    */
   public UpdateAccountTask(TemporaryAccount item, boolean enabled)
   {
      this.item = item;
      this.enabled = enabled;
   }
   
   /**
    * Task executor.
    */
   public void run()
   {
      boolean updated = false;
      
      // Change user account status
      UserDef user = AdminServerImpl.getUser(item.getSubject());
      if (user != null)
      {
         user.enabled = enabled;         
         if (enabled)
         {
            // On enable account change also the password.
            String password = RandomWordGenerator.create(TemporaryAccountPool.GEN_LENGTH, 
                                                         TemporaryAccountPool.GEN_DIGITS, 
                                                         TemporaryAccountPool.GEN_SYMBOLS);
            // Store password in pool.
            item.setPassword(password);
            user.password = HashPassword.hashPassword(item.getPassword());
         }
         // Change account
         updated = AdminServerImpl.setUser(user);         
      }            
      
      if (!updated)
      {
         LOG.logp(Level.WARNING,
                  "UpdateAccountTask.run()",
                  "",
                  "Cannot change account for %s user.",
                  item.getSubject());                     
      }
                        
      // Refresh target.
      AdminServerImpl.targetRefresh();
   }
}
