/*
** Module   : winspawn.c
** Abstract : a simple tool able to spawn a process on an existing user account.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description---------------------------------
** 001 MAG 20131130 First version.
*/

#define _WIN32_WINNT 0x0500

#include <stdio.h>
#include <stdlib.h>
#include <wchar.h>

#include <Windows.h>

#define EXIT_SUCCESS 0

#define ERR_SPAWN_NO_ARGS            -1
#define ERR_SPAWN_LOGON_USER         -2
#define ERR_SPAWN_CREATE_ENVIRONMENT -3
#define ERR_SPAWN_GET_USER_PROFILE   -4
#define ERR_SPAWN_WORKING_DIRECTORY  -5
#define ERR_SPAWN_CREATE_PROCESS     -6
#define ERR_SPAWN_LOAD_LIBRARIES     -7

#define MIN_ARGS 3
#define MAX_LINE 8192

WCHAR szUserProfile[MAX_LINE] = L"";
WCHAR szCommandLine[MAX_LINE] = L"";
WCHAR szPassword[MAX_PATH];

DWORD dwCreationFlags = CREATE_NEW_CONSOLE | CREATE_UNICODE_ENVIRONMENT;
DWORD dwSize = sizeof(szUserProfile)/sizeof(WCHAR);
HANDLE hToken;
LPVOID lpvEnv;

STARTUPINFOW        si = {0};
PROCESS_INFORMATION pi = {0};

/* Function prototypes */
BOOL PathIsRootW(LPWSTR);
BOOL PathAppendW(LPWSTR,LPCWSTR);
BOOL WINAPI CreateEnvironmentBlock(LPVOID*,HANDLE,BOOL);
BOOL WINAPI GetUserProfileDirectoryW(HANDLE,LPWSTR,LPDWORD);

/* Function pointer definitions */
typedef BOOL (fpPathIsRootW)(LPCWSTR);
typedef BOOL (fpPathAppendW)(LPWSTR,LPCWSTR);
typedef BOOL (fpCreateEnvironmentBlock)(LPVOID*,HANDLE,BOOL);
typedef BOOL (fpGetUserProfileDirectoryW)(HANDLE,LPWSTR,LPDWORD);

/* Function pointer instances */
fpPathIsRootW* _PathIsRootW;
fpPathAppendW* _PathAppendW;
fpCreateEnvironmentBlock* _CreateEnvironmentBlock;
fpGetUserProfileDirectoryW* _GetUserProfileDirectoryW;

/* Libraries handles */
HINSTANCE hDllShlwapi, hDllUserenv;

/**
* Load Shlwapi.dll
*
* @return   TRUE  load library success.
*           FALSE error loading library.
*/
BOOL LoadShlwapi()
{
   hDllShlwapi = LoadLibraryW(L"Shlwapi.dll");
   
   if (hDllShlwapi)
   {
      // get functions
      _PathIsRootW = (fpPathIsRootW*) GetProcAddress(hDllShlwapi, "PathIsRootW");
      _PathAppendW = (fpPathAppendW*) GetProcAddress(hDllShlwapi, "PathAppendW");
   }
   
   return hDllShlwapi != NULL;
}

/**
* Load Userenv.dll
*
* @return   TRUE  load library success.
*           FALSE error loading library.
*/
BOOL LoadUserenv()
{
   hDllUserenv = LoadLibraryW(L"Userenv.dll");
   
   if (hDllUserenv)
   {
      // get functions
      _CreateEnvironmentBlock = (fpCreateEnvironmentBlock*) 
            GetProcAddress(hDllUserenv, "CreateEnvironmentBlock");
      _GetUserProfileDirectoryW = (fpGetUserProfileDirectoryW*) 
            GetProcAddress(hDllUserenv, "GetUserProfileDirectoryW");
   }
   
   return hDllUserenv != NULL;
}

/**
* Clean up.
*/
void CleanUp()
{
   /* Close handles */
   CloseHandle(hToken);
   CloseHandle(pi.hProcess);
   CloseHandle(pi.hThread);
   /* Unload Shlwapi.dll */
   if (!hDllShlwapi)
   {
      FreeLibrary(hDllShlwapi);
   }
   /* Unload Userenv.dll */   
   if (!hDllUserenv)
   {
      FreeLibrary(hDllUserenv);
   }
}

/**
* Display error messages from system.
*
* @param   pszAPI
*          Name of the API call that generate the error.
*/
void DisplayError(LPCWSTR pszAPI)
{
   LPVOID lpvMessageBuffer;

   FormatMessageW(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                  NULL,
                  GetLastError(),
                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                  (LPWSTR)&lpvMessageBuffer,
                  0,
                  0);

   wprintf(L"ERROR: API      = %s.\n", pszAPI);
   wprintf(L"      error code = %d.\n", GetLastError());
   wprintf(L"      message   = %s.\n", (LPWSTR)lpvMessageBuffer);

   LocalFree(lpvMessageBuffer);

   CleanUp();
}

/**
* Prepare a command line string from arguments.
*
* @param   argc
*          Number of command line arguments.
* @param   argv
*          Command line arguments.
*
* @return  Command line string.
*/
LPWSTR CommandLine(int argc, LPWSTR argv[])
{
   int i;
   for (i = MIN_ARGS;i < argc;i++)
   {
      if (i != MIN_ARGS)
      {
         wcscat(szCommandLine, L" ");
      }
      wcscat(szCommandLine, argv[i]);
   }

   return szCommandLine;
}

/**
* Read user password from stdin.
*
* @return  User password.
*/
LPWSTR ReadPassword()
{
   return _getws(szPassword);
}

/**
* Main function.
* MinGW does not support UNICODE wmain function.
* A main function is used as a wrapper for wmain.
*
* @param   argc
*          Number of command line arguments.
* @param   argv
*          Command line arguments.
*
* @return  Exit status.
*/
int wmain(int argc, LPWSTR argv[])
{
   si.cb = sizeof(STARTUPINFO);

   if(argc < MIN_ARGS)
   {
      wprintf(L"Usage: %s <user@domain> <workingdir> <command> [args]\n", argv[0]);
      return ERR_SPAWN_NO_ARGS;
   }

   /* Get user password */
   LPWSTR pswd = ReadPassword();

   /* Attempts to log a user on to the local computer. */
   if (!LogonUserW(argv[1],
                   0,
                   pswd,
                   LOGON32_LOGON_INTERACTIVE,
                   LOGON32_PROVIDER_DEFAULT,
                   &hToken))
   {
      DisplayError(L"LogonUser");
      return ERR_SPAWN_LOGON_USER;
   }

   /* Retrieves the environment variables for the specified user. */
   if (!_CreateEnvironmentBlock(&lpvEnv, hToken, TRUE))
   {
      DisplayError(L"CreateEnvironmentBlock");
      return ERR_SPAWN_CREATE_ENVIRONMENT;
   }

   /* Retrieves the path to the root directory of the specified user's profile. */
   if (!_GetUserProfileDirectoryW(hToken, szUserProfile, &dwSize))
   {
      DisplayError(L"GetUserProfileDirectory");
      return ERR_SPAWN_GET_USER_PROFILE;
   }

   /* Working directory */
   LPWSTR szWorkingDir = argv[2];

   /* If relative path to user profile root directory */
   if (!_PathIsRootW(szWorkingDir))
   {
      if (!_PathAppendW(szUserProfile, szWorkingDir))
      {
         DisplayError(L"PathAppend");
         return ERR_SPAWN_WORKING_DIRECTORY;
      }
      szWorkingDir = szUserProfile;
   }

   /* Command line */
   LPWSTR command = CommandLine(argc, argv);

   /* Creates a new process and its primary thread.
      Then the new process runs the specified executable file in the security context
      of the specified credentials (user, domain, and password).
      It can optionally load the user profile for a specified user. */

   if (!CreateProcessWithLogonW(argv[1],              // user name
                                0,                    // domain
                                pswd,                 // password
                                LOGON_WITH_PROFILE,   // logon options
                                0,                    // application name
                                command,              // command line
                                dwCreationFlags,      // flags
                                lpvEnv,               // environment block
                                szWorkingDir,         // current dir
                                &si,                  // startup info
                                &pi))                 // process info
   {
      DisplayError(L"CreateProcessWithLogonW");
      return ERR_SPAWN_CREATE_PROCESS;
   }

   /* Close handles */
   CleanUp();

   return EXIT_SUCCESS;
}

/**
 * This is for the MinGW compiler which does not support wmain.
 * It is a wrapper for _tmain when _UNICODE is defined (wmain).
 * This wrapper adds ~300 bytes to the program and negligible overhead.
 */

#undef _tmain
#ifdef _UNICODE
#define _tmain wmain
#else
#define _tmain main
#endif

#if defined(__GNUC__) && defined(_UNICODE)

#ifndef __MSVCRT__
#error Unicode main function requires linking to MSVCRT
#endif

extern int _CRT_glob;
extern
#ifdef __cplusplus
"C"
#endif
void __wgetmainargs(int*, LPWSTR**, LPWSTR**, int, int*);

int main() 
{
   LPWSTR *enpv, *argv;
   int argc, si = 0;
   // this also creates the global variable __wargv
   __wgetmainargs(&argc, &argv, &enpv, _CRT_glob, &si); 
   /* Load DLL libraries. */
   if (!LoadShlwapi() || !LoadUserenv())
   {
      DisplayError(L"LoadLibraryW");
      return ERR_SPAWN_LOAD_LIBRARIES;
   }
   else
   {
#ifdef MAIN_USE_ENVP
      return wmain(argc, argv, enpv);
#else
      return wmain(argc, argv);
#endif
   }
}

#endif //defined(__GNUC__) && defined(_UNICODE)
